@@ -3,7 +3,7 @@
  *
  *             refine AIM
  *
- *      Copyright 2014-2025, Massachusetts Institute of Technology
+ *      Copyright 2014-2026, Massachusetts Institute of Technology
  *      Licensed under The GNU Lesser General Public License, version 2.1
  *      See http://www.opensource.org/licenses/lgpl-2.1.php
  *
@@ -265,6 +265,147 @@ static int destroy_aimStorage(aimStorage *refineInstance)
 }
 
 
+// HACK!! Replace the CAD geometry with refine's interpretation of ByteFlow (differs from libMeshb)
+static int
+refine_ByteFlow(void *aimInfo)
+{
+  int status = CAPS_SUCCESS;
+
+  char refine_in[PATH_MAX];
+  FILE *fp = NULL;
+
+  int int_code, int_version, keyword_code;
+  long position=0, next_position=0, end_position=0, byteflow_position=0;
+  int32_t temp_int;
+  int64_t temp_long;
+  size_t nbytes;
+  long int_size=0, fp_size=0, header_size=0;
+
+  snprintf(refine_in, PATH_MAX, "%s%s", refine_in_pre, MESHEXTENSION);
+  fp = aim_fopen(aimInfo, refine_in, "rb+");
+  if (fp == NULL) {
+    AIM_ERROR(aimInfo, "Cannot open %s", refine_in);
+    status = CAPS_IOERR;
+    goto cleanup;
+  }
+
+  status = fread((unsigned char *)&int_code, 4, 1, fp);
+  if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+  if (int_code != 1) {
+    AIM_ERROR(aimInfo, "libMeshb code != 1!!");
+    status = CAPS_IOERR;
+    goto cleanup;
+  }
+  status = fread((unsigned char *)&int_version, 4, 1, fp);
+  if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+  if (int_version < 1 || 4 < int_version) {
+    AIM_ERROR(aimInfo, "libMeshb version %d not supported!", int_version);
+    status = CAPS_IOERR;
+    goto cleanup;
+  }
+
+  next_position = ftell(fp);
+  status = fseek(fp, 0, SEEK_END);
+  if (status != 0) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+  end_position = ftell(fp);
+  while (next_position <= end_position && 0 != next_position) {
+    position = next_position;
+    status = fseek(fp, position, SEEK_SET);
+    if (status != 0) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+
+    status = fread((unsigned char *)&keyword_code, 4, 1, fp);
+    if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+
+    if (byteflow_position > 0 && keyword_code != GmfEnd) {
+      AIM_ERROR(aimInfo, "GmfByteFlow must be the last entry in %s", refine_in);
+      status = CAPS_IOERR;
+      goto cleanup;
+    }
+
+    if (3 <= int_version) {
+      status = fread(&temp_long, sizeof(temp_long), 1, fp);
+      if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+      next_position = temp_long;
+    } else {
+      status = fread(&temp_int, sizeof(temp_int), 1, fp);
+      if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+      next_position = temp_int;
+    }
+    if (keyword_code == GmfByteFlow) {
+      byteflow_position = position;
+
+      // read the size of the byte stream in the file
+      if (3 <= int_version) {
+        status = fread(&temp_long, sizeof(temp_long), 1, fp);
+        if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+        nbytes = temp_long;
+      } else {
+        status = fread(&temp_int, sizeof(temp_int), 1, fp);
+        if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+        nbytes = temp_int;
+      }
+    }
+  }
+
+  end_position = position;
+
+  // no geometry to fix
+  if (byteflow_position == 0) goto cleanup;
+
+  // good to go if no padding
+  if (end_position - byteflow_position == nbytes) goto cleanup;
+
+  // update the nex_position without integer padding
+  int_size = 4;
+  fp_size = 4;
+  if (2 < int_version) fp_size = 8;
+  if (3 < int_version) int_size = 8;
+  header_size = 4 + fp_size + int_size;
+
+  next_position = header_size + nbytes + byteflow_position;
+
+  status = fseek(fp, byteflow_position + int_size, SEEK_SET);
+  if (status != 0) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+
+  // update the next position
+  if (3 <= int_version) {
+    temp_long = next_position;
+    status = fwrite(&temp_long, sizeof(temp_long), 1, fp);
+    if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+  } else {
+    temp_int = next_position;
+    status = fwrite(&temp_int, sizeof(temp_int), 1, fp);
+    if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+  }
+
+  // jump pas the stream data
+  status = fseek(fp, next_position, SEEK_SET);
+  if (status != 0) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+
+  // Write the end statement
+  if (3 <= int_version) {
+    temp_long = GmfEnd;
+    status = fwrite(&temp_long, sizeof(temp_long), 1, fp);
+    if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+    temp_long = 0;
+    status = fwrite(&temp_long, sizeof(temp_long), 1, fp);
+    if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+  } else {
+    temp_int = GmfEnd;
+    status = fwrite(&temp_int, sizeof(temp_int), 1, fp);
+    if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+    temp_int = 0;
+    status = fwrite(&temp_int, sizeof(temp_int), 1, fp);
+    if (status != 1) { status = CAPS_IOERR; AIM_STATUS(aimInfo, status); }
+  }
+
+  status = CAPS_SUCCESS;
+cleanup:
+  if (fp != NULL) fclose(fp);
+  return status;
+}
+
+
 static int
 refine_genMultiScale(void *aimInfo)
 {
@@ -319,6 +460,10 @@ refine_genMultiScale(void *aimInfo)
   status = aim_rmFile(aimInfo, refine_out);
   AIM_STATUS(aimInfo, status);
 
+  // HACK! Remove the ByteFlow padding in the libMeshb file for refine
+  status = refine_ByteFlow(aimInfo);
+  AIM_STATUS(aimInfo, status);
+
   if (ScalarFieldFile->nullVal == NotNull) {
 
     if (access(ScalarFieldFile->vals.string, F_OK) != 0) {
@@ -451,7 +596,6 @@ cleanup:
   return status;
 }
 
-
 // ********************** AIM Function Break *****************************
 /* aimInputs: Input Information for the AIM */
 int
@@ -880,7 +1024,7 @@ aimPreAnalysis(/*@unused@*/ const void *instStore, void *aimInfo,
     AIM_STATUS(aimInfo, status);
 
   } else {
-
+  
     // Compute the multiscale metric
     status = refine_genMultiScale(aimInfo);
     AIM_STATUS(aimInfo, status);
@@ -931,7 +1075,8 @@ aimPreAnalysis(/*@unused@*/ const void *instStore, void *aimInfo,
     if (inputs[inFun3D-1].vals.integer == (int) false) {
 
       fprintf(fp,
-              " adapt refine_in%s --metric %s --egads %s -s %d -x %s%s",
+              " adapt %s%s --metric %s --egads %s -s %d -x %s%s",
+              refine_in_pre,
               MESHEXTENSION,
               metricFileName,
               egadsFileName,
@@ -945,7 +1090,8 @@ aimPreAnalysis(/*@unused@*/ const void *instStore, void *aimInfo,
       AIM_STATUS(aimInfo, status);
 
       fprintf(fp,
-              " loop refine_in %s %le --egads %s  -s %d",
+              " loop %s %s %le --egads %s  -s %d",
+              refine_in_pre,
               refine_out_pre,
               inputs[inComplexity-1].vals.real,
               egadsFileName,
